@@ -12,7 +12,7 @@ The simple local key-value store.
 - Small db file size.
 - Separated files. (data record file and index file)
 - One database has some db-map-string and some db-map-u64.
-- minimum support rustc 1.54.0 (a178d0322 2021-07-26)
+- minimum support rustc rustc 1.53.0 (53cb7b09b 2021-06-17)
 
 # Compatibility
 
@@ -23,10 +23,10 @@ The simple local key-value store.
 ## Example DbMapString:
 
 ```
-use siamesedb::DbMapString;
+use siamesedb::{DbMapString, DbXxx};
 
 fn main() -> std::io::Result<()> {
-    let db_name = "target/tmp/doc-test1.shamdb";
+    let db_name = "target/tmp/doc-test1.siamesedb";
     // remove database
     let _ = std::fs::remove_dir_all(db_name);
     // create or open database
@@ -36,7 +36,7 @@ fn main() -> std::io::Result<()> {
     //
     let r = db_map.get_string("key1")?;
     assert_eq!(r, None);
-    db_map.put_string("key1", "value1")?;
+    db_map.put_string("key1".to_string(), "value1")?;
     let r = db_map.get_string("key1")?;
     assert_eq!(r, Some("value1".to_string()));
     db_map.sync_data()?;
@@ -47,19 +47,19 @@ fn main() -> std::io::Result<()> {
 ## Example DbMapU64:
 
 ```
-use siamesedb::DbMapU64;
+use siamesedb::{DbMapU64, DbXxx};
 
 fn main() -> std::io::Result<()> {
-    let db_name = "target/tmp/doc-test2.shamdb";
+    let db_name = "target/tmp/doc-test2.siamesedb";
     // remove database
     let _ = std::fs::remove_dir_all(db_name);
     // create or open database
     let db = siamesedb::open_file(db_name)?;
     let mut db_map = db.db_map_u64("some_list1")?;
-    let r = db_map.get_string(120)?;
+    let r = db_map.get_string(&120)?;
     assert_eq!(r, None);
     db_map.put_string(120, "value120")?;
-    let r = db_map.get_string(120)?;
+    let r = db_map.get_string(&120)?;
     assert_eq!(r, Some("value120".to_string()));
     db_map.sync_data()?;
     Ok(())
@@ -67,6 +67,7 @@ fn main() -> std::io::Result<()> {
 ```
 
 */
+use std::borrow::Borrow;
 use std::io::Result;
 use std::path::Path;
 
@@ -83,90 +84,24 @@ pub fn open_file<P: AsRef<Path>>(path: P) -> Result<filedb::FileDb> {
     filedb::FileDb::open(path)
 }
 
-/// key-value map store interface. the key type is `&str`.
-pub trait DbMapString {
-    /// returns the value corresponding to the key.
-    fn get(&mut self, key: &str) -> Result<Option<Vec<u8>>>;
-
-    /// inserts a key-value pair into the db-map.
-    fn put(&mut self, key: &str, value: &[u8]) -> Result<()>;
-
-    /// removes a key from the db-map.
-    fn delete(&mut self, key: &str) -> Result<()>;
-
-    /// flush file buffer, the dirty intermediate buffered content is written
-    fn flush(&mut self) -> Result<()>;
-
-    /// synchronize all OS-internal metadata to storage.
-    fn sync_all(&mut self) -> Result<()>;
-
-    /// synchronize data to storage, except file metadabe.
-    fn sync_data(&mut self) -> Result<()>;
-
-    /// returns true if the map contains a value for the specified key.
-    fn has_key(&mut self, key: &str) -> Result<bool> {
-        self.get(key).map(|opt| opt.is_some())
-    }
-
-    /// returns the value corresponding to the key. the value is converted to `String`.
-    fn get_string(&mut self, key: &str) -> Result<Option<String>> {
-        self.get(key)
-            .map(|opt| opt.map(|val| String::from_utf8_lossy(&val).to_string()))
-    }
-
-    /// inserts a key-value pair into the db-map. the value is `&str` and it is converted to `&[u8]`
-    fn put_string(&mut self, key: &str, value: &str) -> Result<()> {
-        self.put(key, value.as_bytes())
-    }
-}
-
-/// key-value list store interface. the key type is `u64`.
-pub trait DbMapU64 {
-    /// returns the value corresponding to the key.
-    fn get(&mut self, key: u64) -> Result<Option<Vec<u8>>>;
-
-    /// inserts a key-value pair into the db-list.
-    fn put(&mut self, key: u64, value: &[u8]) -> Result<()>;
-
-    /// removes a key from the db-list.
-    fn delete(&mut self, key: u64) -> Result<()>;
-
-    /// flush file buffer, the dirty intermediate buffered content is written
-    fn flush(&mut self) -> Result<()>;
-
-    /// synchronize all OS-internal metadata to storage.
-    fn sync_all(&mut self) -> Result<()>;
-
-    /// synchronize data to storage, except file metadabe.
-    fn sync_data(&mut self) -> Result<()>;
-
-    /// returns true if the list contains a value for the specified key.
-    fn has_key(&mut self, key: u64) -> Result<bool> {
-        self.get(key).map(|opt| opt.is_some())
-    }
-
-    /// returns the value corresponding to the key. the value is converted to `String`.
-    fn get_string(&mut self, key: u64) -> Result<Option<String>> {
-        self.get(key)
-            .map(|opt| opt.map(|val| String::from_utf8_lossy(&val).to_string()))
-    }
-
-    /// inserts a key-value pair into the db-list. the value is `&str` and it is converted to `&[u8]`
-    fn put_string(&mut self, key: u64, value: &str) -> Result<()> {
-        self.put(key, value.as_bytes())
-    }
-}
-
 /// generic key-value map store interface. the key type is `KT`.
 pub trait DbXxx<KT> {
     /// returns the value corresponding to the key.
-    fn get(&mut self, key: &KT) -> Result<Option<Vec<u8>>>;
+    fn get<Q>(&mut self, key: &Q) -> Result<Option<Vec<u8>>>
+    where
+        KT: Borrow<Q> + Ord,
+        Q: Ord + ?Sized;
 
     /// inserts a key-value pair into the db.
-    fn put(&mut self, key: &KT, value: &[u8]) -> Result<()>;
+    fn put(&mut self, key: KT, value: &[u8]) -> Result<()>
+    where
+        KT: Ord;
 
     /// removes a key from the db.
-    fn delete(&mut self, key: &KT) -> Result<()>;
+    fn delete<Q>(&mut self, key: &Q) -> Result<()>
+    where
+        KT: Borrow<Q> + Ord,
+        Q: Ord + ?Sized;
 
     /// flush file buffer, the dirty intermediate buffered content is written
     fn flush(&mut self) -> Result<()>;
@@ -178,18 +113,35 @@ pub trait DbXxx<KT> {
     fn sync_data(&mut self) -> Result<()>;
 
     /// returns true if the map contains a value for the specified key.
-    fn has_key(&mut self, key: &KT) -> Result<bool> {
+    fn has_key<Q>(&mut self, key: &Q) -> Result<bool>
+    where
+        KT: Borrow<Q> + Ord,
+        Q: Ord + ?Sized,
+    {
         self.get(key).map(|opt| opt.is_some())
     }
 
     /// returns the value corresponding to the key. the value is converted to `String`.
-    fn get_string(&mut self, key: &KT) -> Result<Option<String>> {
+    fn get_string<Q>(&mut self, key: &Q) -> Result<Option<String>>
+    where
+        KT: Borrow<Q> + Ord,
+        Q: Ord + ?Sized,
+    {
         self.get(key)
             .map(|opt| opt.map(|val| String::from_utf8_lossy(&val).to_string()))
     }
 
     /// inserts a key-value pair into the db-map. the value is `&str` and it is converted to `&[u8]`
-    fn put_string(&mut self, key: &KT, value: &str) -> Result<()> {
+    fn put_string(&mut self, key: KT, value: &str) -> Result<()>
+    where
+        KT: Ord,
+    {
         self.put(key, value.as_bytes())
     }
 }
+
+/// key-value map store interface. the key type is `String`.
+pub trait DbMapString: DbXxx<String> {}
+
+/// key-value map store interface. the key type is `u64`.
+pub trait DbMapU64: DbXxx<u64> {}
